using System;
using System.Windows;
using System.Windows.Interactivity;

namespace FinalDemo.Framework
{
	public class AttachViewModel : Behavior<FrameworkElement>
	{
		private ViewModelBase _viewModel;
		protected override void OnAttached()
		{
			base.OnAttached();

			_viewModel = LocateViewModel(AssociatedObject);
			AssociatedObject.DataContext = _viewModel;

			AssociatedObject.Loaded += AssociatedObjectLoaded;
		}

		protected override void OnDetaching()
		{

			IViewAware viewModel = _viewModel as IViewAware;
			Window view = AssociatedObject as Window;

			if (viewModel != null && view != null)
			{
				viewModel.CloseView -= view.Close;
				viewModel.CloseDialogView -= CloseDialog;
			}

			AssociatedObject.Loaded -= AssociatedObjectLoaded;
			base.OnDetaching();
		}

		void AssociatedObjectLoaded(object sender, RoutedEventArgs e)
		{
			_viewModel.AutoWireCommands(AssociatedObject);

			IViewAware viewModel = _viewModel as IViewAware;
			Window view = AssociatedObject as Window;

			if (viewModel != null && view != null)
			{
				view.Activated += (o, v) => viewModel.ViewActivated();
				view.Deactivated += (o, v) => viewModel.ViewDeactivated();
				
				viewModel.CloseView += view.Close;
				viewModel.CloseDialogView += CloseDialog;
			}
		}

		private void CloseDialog(bool dialogResult)
		{
			Window view = AssociatedObject as Window;
			if (view != null)
			{
				view.DialogResult = dialogResult;
				view.Close();
			};
		}

		public static ViewModelBase LocateViewModel(object view)
		{
			var viewName = view.GetType().FullName;
			if (String.IsNullOrEmpty(viewName))
				return null;

			var viewModelTypeName = viewName.Replace("View", "ViewModel");
			var viewModelType = Type.GetType(viewModelTypeName);
			
			return Activator.CreateInstance(viewModelType) as ViewModelBase;

		}

	}

}