using System;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Input;

namespace FinalDemo.Framework
{
	public class AutoWiredCommand : ICommand
	{
		private readonly PropertyInfo _canExecute;
		private readonly MethodInfo _execute;
		private readonly object _viewModel;

		public event EventHandler CanExecuteChanged = delegate { };

		public AutoWiredCommand(object viewModel, MethodInfo execute, PropertyInfo canExecute)
		{
			_viewModel = viewModel;
			_execute = execute;
			_canExecute = canExecute;

			var notifier = _viewModel as INotifyPropertyChanged;
			if (notifier != null && _canExecute != null)
			{
				notifier.PropertyChanged += (s, e) =>
				{
					if (e.PropertyName == _canExecute.Name)
						CanExecuteChanged(this, EventArgs.Empty);
				};
			}

		}

		public bool CanExecute(object parameter)
		{
			if (_canExecute != null)
				return (bool)_canExecute.GetValue(_viewModel, null);

			return true;
		}

		public void Execute(object parameter)
		{
			_execute.Invoke(_viewModel, null);
		}
		
	}
}