using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Controls.Primitives;
using System.Windows.Data;

namespace FinalDemo.Framework
{
	public class ViewModelBase : INotifyPropertyChanged
	{
		#region INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		public void NotifyPropertyChanged(string propertyName)
		{
			PropertyChangedEventHandler handler = PropertyChanged;
			if (handler != null)
			{
				var e = new PropertyChangedEventArgs(propertyName);
				handler(this, e);
			}
		}

		#endregion // INotifyPropertyChanged

		public void AutoWireCommands(FrameworkElement view)
		{
			var viewModelType = GetType();
			var viewModelMethods = viewModelType.GetMethods();
			var viewModelProperties = viewModelType.GetProperties();

			foreach (MethodInfo method in viewModelMethods)
			{
				string methodName = method.Name;
				var foundControl = view.FindName(method.Name) as DependencyObject;
				if (foundControl == null)
					continue;

				DependencyProperty property;
				if (CanBind(foundControl, out property) && property != null)
				{
					var propertyInfo = viewModelProperties.FirstOrDefault(x => x.Name == "Can" + methodName);

					MethodInfo methodInfo = method;
					AutoWiredCommand command = new AutoWiredCommand(this, methodInfo, propertyInfo);
					BindingOperations.SetBinding(foundControl, property, new Binding { Source = command });
				}
			}
		}

		protected virtual bool CanBind(DependencyObject obj, out DependencyProperty property)
		{
			property = ButtonBase.CommandProperty;
			return (obj is ButtonBase);
		}

		private PropertyMonitor _propertyMonitor;
		public ViewModelBase()
		{
			_propertyMonitor = new PropertyMonitor(this);
		}

	}
}