﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using FinalDemo.Assets.Win32;
using FinalDemo.ViewModel;

namespace FinalDemo.View
{
	public partial class MainView : Window
	{
		public MainView()
		{
			InitializeComponent();
			Loaded += MainWindowLoaded;
		}

		#region Style

		private void MainWindowLoaded(object sender, RoutedEventArgs e)
		{
			ApplyStyle(sender);
		}

		private void Header_PreviewMouseDown(object sender, MouseButtonEventArgs e)
		{
			if (DateTime.Now.Subtract(_headerLastClick) <= _doubleClickSpan)
			{
				WindowState = (WindowState == WindowState.Normal) ? WindowState.Maximized : WindowState.Normal;
			}
			_headerLastClick = DateTime.Now;

			if (Mouse.LeftButton == MouseButtonState.Pressed)
				DragMove();
		}

		private void ApplyStyle(object sender)
		{
			_hwndSource = PresentationSource.FromVisual((Visual)sender) as HwndSource;

			if (_hwndSource != null)
				_hwndSource.AddHook(WndProc);
		}

		public void ApplyShadow()
		{
			if (Win32.IsDwmAvailable())
			{
				Win32.MARGINS margins;
				margins.bottomHeight = 1;
				margins.leftWidth = 0;
				margins.rightWidth = 0;
				margins.topHeight = 0;

				var helper = new WindowInteropHelper(this);
				Win32.DwmExtendFrameIntoClientArea(helper.Handle, ref margins);
			}
			_shadowApplicationAttempted = true;
		}

		private IntPtr WndProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
		{
			switch (msg)
			{
				case 0x31e:
					ApplyShadow();
					handled = true;
					break;
				case 0x86:
					{
						IntPtr ptr = Win32.DefWindowProc(hwnd, msg, wParam, new IntPtr(-1));
						handled = true;
						return ptr;
					}
				case 0x83:
					if (wParam == new IntPtr(1))
					{
						handled = true;
					}
					break;

				case 20:
					System.Drawing.Graphics.FromHdc(wParam).Clear(System.Drawing.Color.White);
					handled = true;
					break;

				case 0x24:
					Win32Api.WmGetMinMaxInfo(hwnd, lParam);
					handled = true;
					break;

				case 70:
					if (!_shadowApplicationAttempted)
					{
						ApplyShadow();
					}
					break;
			}
			return IntPtr.Zero;
		}

		private readonly static TimeSpan _doubleClickSpan = TimeSpan.FromMilliseconds(500);
		private DateTime _headerLastClick = DateTime.MinValue;
		private HwndSource _hwndSource;
		private bool _shadowApplicationAttempted;

		#endregion // Style

	}
}
