using System;
using System.Threading;
using FinalDemo.Framework;
using FinalDemo.Services;

namespace FinalDemo.ViewModel
{
	public class MainViewModel : ViewModelBase, IViewAware
	{

		private string _taskName = String.Empty;
		public string TaskName
		{
			get { return _taskName; }
			set
			{
				_taskName = value;
				NotifyPropertyChanged("TaskName");
			}
		}

		private const string DefaultTime = "0:00";
		private string _taskTimer = DefaultTime;
		public string TaskTimer
		{
			get { return _taskTimer; }
			set
			{
				_taskTimer = value;
				NotifyPropertyChanged("TaskTimer");
			}
		}

		private readonly TimeService _timeService;

		public MainViewModel()
		{
			_timeService = new TimeService();
			_timeService.Timer += OnTimer;
		}

		#region Event Handlers
		
		private void OnTimer(int seconds)
		{
			TaskTimer = String.Format("{0}:{1:D2}", seconds / 60, seconds % 60);
		}

		public void ButtonAdd()
		{
			string newTaskName = DialogServiceLocator.TaskDialogService.Show();

			if (!string.IsNullOrEmpty(newTaskName))
			{
				TaskName = newTaskName;
				_timeService.Start();
			}
		}

		public void ButtonCancel()
		{
			_timeService.Cancel();
			TaskName = String.Empty;
			TaskTimer = DefaultTime;
		}

		[DependsOn("TaskName")]
		public bool CanButtonCancel
		{
			get
			{
				return TaskName != String.Empty;	
			}
			
		}

		public void ButtonClose()
		{
			var handler = CloseView;
			if (handler != null)
				handler();
		}

		#endregion //Event Handlers

		#region Implementation of IViewAware

		public void ViewActivated()
		{
			// handle Activated event
		}

		public void ViewDeactivated()
		{
			// handle Deactivated event
		}

		public event Action CloseView;
		public event Action<bool> CloseDialogView;

		#endregion

		/*
		private bool _visible;
		public bool Visible
		{
			get { return _visible; }
			set
			{
				_visible = value;
				NotifyPropertyChanged("Property1");
				NotifyPropertyChanged("Property2");
				NotifyPropertyChanged("Property3");
				NotifyPropertyChanged("Property4");
			}
		}
		*/
	}

	#region TimeService

	public class TimeService
	{
		public event Action<int> Timer;

		private readonly Timer _timer;
		private DateTime _startTime;

		public TimeService()
		{
			_timer = new Timer(OnTimer, null, Timeout.Infinite, Timeout.Infinite);
		}

		private void OnTimer(object state)
		{
			var handler = Timer;
			if (handler != null)
				handler((int)(DateTime.Now - _startTime).TotalSeconds);
		}

		public void Start()
		{
			_startTime = DateTime.Now;
			_timer.Change(0, 1000);
		}

		public void Cancel()
		{
			_timer.Change(Timeout.Infinite, Timeout.Infinite);
		}
	}

	#endregion //TimeService

}